#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup


setup(name='vf',
      version='0.0.1rc0',
      description='VF: Validation Functions (for Python dicts.)',
      author='Polydojo, Inc.',
      author_email='8079593+sumukhbarve@users.noreply.github.com',
      url='https://github.com/polydojo/vf',
      py_modules=['vf'],
     )
